@@ -9,10 +9,9 @@
 use super::Core;
 use crate::messaging::{
     system::{KeyedSig, SectionAuth, SystemMsg},
-    MessageType, SrcLocation, WireMsg,
+    MessageId, MessageType, SrcLocation, WireMsg,
 };
 use crate::routing::{
-    dkg::SectionAuthUtils,
     error::{Error, Result},
     log_markers::LogMarker,
     messages::WireMsgUtils,
@@ -65,11 +64,17 @@ impl Core {
             )
             .await?;
 
-        self.fire_node_event_for_any_new_adults().await?;
+        let mut commands = vec![];
+
+        commands.extend(self.fire_node_event_for_any_new_adults().await?);
 
         // always run this, only changes will trigger events
-        self.update_self_for_new_node_state_and_fire_events(snapshot)
-            .await
+        commands.extend(
+            self.update_self_for_new_node_state_and_fire_events(snapshot)
+                .await?,
+        );
+
+        Ok(commands)
     }
 
     pub(crate) async fn handle_anti_entropy_retry_msg(
@@ -80,16 +85,132 @@ impl Core {
         bounced_msg: Bytes,
         sender: Peer,
     ) -> Result<Vec<Command>> {
-        let bounced_msg = match WireMsg::deserialize(bounced_msg)? {
-            MessageType::System { msg, .. } => msg,
-            _ => {
-                warn!("Non System MessageType received at Node in AE response. We do not handle any other type yet");
-                return Ok(vec![]);
-            }
-        };
+        let dst_section_key = section_auth.section_key();
         let snapshot = self.state_snapshot().await;
 
-        info!("Anti-Entropy: retry message received from peer: {}", sender);
+        let to_resend = self
+            .update_network_knowledge(
+                section_auth,
+                section_signed,
+                proof_chain,
+                bounced_msg,
+                sender.clone(),
+            )
+            .await?;
+
+        match to_resend {
+            None => Ok(vec![]),
+            Some((msg_to_resend, _)) => {
+                // TODO: we may need to check if 'bounced_msg' dest section pk is different
+                // from the received new SAP key, to prevent from endlessly resending a msg
+                // if a sybil/corrupt peer keeps sending us the same AE msg.
+                trace!("{}", LogMarker::AeResendAfterRetry);
+
+                self.create_or_wait_for_backoff(&sender).await;
+
+                let mut result = Vec::new();
+                if let Ok(cmds) = self
+                    .update_self_for_new_node_state_and_fire_events(snapshot)
+                    .await
+                {
+                    result.extend(cmds);
+                }
+
+                result.push(
+                    self.send_direct_message(sender, msg_to_resend, dst_section_key)
+                        .await?,
+                );
+
+                Ok(result)
+            }
+        }
+    }
+
+    pub(crate) async fn handle_anti_entropy_redirect_msg(
+        &self,
+        section_auth: SectionAuthorityProvider,
+        section_signed: KeyedSig,
+        section_chain: SecuredLinkedList,
+        bounced_msg: Bytes,
+        sender: Peer,
+    ) -> Result<Vec<Command>> {
+        let dst_section_key = section_auth.section_key();
+
+        // We choose the Elder closest to the dest section key,
+        // just to pick one of them in an arbitrary but deterministic fashion.
+        let target_name = XorName::from(PublicKey::Bls(dst_section_key));
+        let chosen_dst_elder = section_auth
+            .elders()
+            .sorted_by(|lhs, rhs| target_name.cmp_distance(&lhs.name(), &rhs.name()))
+            .peekable()
+            .peek()
+            .map(|elder| (*elder).clone());
+
+        let to_resend = self
+            .update_network_knowledge(
+                section_auth,
+                section_signed,
+                section_chain,
+                bounced_msg,
+                sender.clone(),
+            )
+            .await?;
+
+        match to_resend {
+            None => Ok(vec![]),
+            Some((msg_to_redirect, msg_id)) => match chosen_dst_elder {
+                None => {
+                    error!(
+                            "Failed to find closest Elder to resend msg ({:?}) upon AE-Redirect response.",
+                            msg_id
+                        );
+                    Ok(vec![])
+                }
+                Some(elder) if elder.addr() == sender.addr() => {
+                    error!(
+                            "Failed to find an alternative Elder to resend msg ({:?}) upon AE-Redirect response.",
+                            msg_id
+                        );
+                    Ok(vec![])
+                }
+                Some(elder) => {
+                    trace!("{}", LogMarker::AeResendAfterAeRedirect);
+
+                    self.create_or_wait_for_backoff(&elder).await;
+
+                    let cmd = self
+                        .send_direct_message(elder, msg_to_redirect, dst_section_key)
+                        .await?;
+
+                    Ok(vec![cmd])
+                }
+            },
+        }
+    }
+
+    // Try to update network knowledge and return the 'SystemMsg' that needs to be resent.
+    async fn update_network_knowledge(
+        &self,
+        section_auth: SectionAuthorityProvider,
+        section_signed: KeyedSig,
+        proof_chain: SecuredLinkedList,
+        bounced_msg: Bytes,
+        sender: Peer,
+    ) -> Result<Option<(SystemMsg, MessageId)>> {
+        let (bounced_msg, msg_id, dst_location) = match WireMsg::deserialize(bounced_msg)? {
+            MessageType::System {
+                msg,
+                msg_id,
+                dst_location,
+                ..
+            } => (msg, msg_id, dst_location),
+            _ => {
+                warn!("Non System MessageType received in AE response. We do not handle any other type in AE msgs yet.");
+                return Ok(None);
+            }
+        };
+
+        info!("Anti-Entropy: message received from peer: {}", sender);
 
         // If we have the key share for new SAP key we can switch to this new SAP.
         // Otherwise, only update when not being the elder of new SAP.
@@ -101,13 +222,15 @@ impl Core {
                 .await
                 .is_ok();
 
-        // update our network knowledge.
+        let prefix = section_auth.prefix();
+        let dst_section_key = section_auth.section_key();
         let signed_sap = SectionAuth {
             value: section_auth.clone(),
             sig: section_signed,
         };
 
-        let updated = self
+        // Update our network knowledge.
+        if self
             .network_knowledge
             .update_knowledge_if_valid(
                 signed_sap,
@@ -116,143 +239,26 @@ impl Core {
                 &self.node.read().await.name(),
                 switch_to_new_sap,
             )
-            .await?;
-
-        let mut result = Vec::new();
-        if updated {
-            self.write_prefix_map().await;
-            info!("PrefixMap written to disk");
-
-            // Regardless if the SAP already existed, as long as it was valid
-            // (it may have been just updated by a concurrent handler of another bounced msg),
-            // we still resend this message.
-            //
-            // TODO: we may need to check if 'bounced_msg' dest section pk is different
-            // from the received new SAP key, to prevent from endlessly resending a msg
-            // if a sybil/corrupt peer keeps sending us the same AE msg.
-            let dst_section_pk = section_auth.section_key();
-            trace!("{}", LogMarker::AeResendAfterRetry);
-
-            self.create_or_wait_for_backoff(&sender).await;
-
-            if let Ok(cmds) = self
-                .update_self_for_new_node_state_and_fire_events(snapshot)
-                .await
-            {
-                result.extend(cmds);
-            }
-            result.push(
-                self.send_direct_message(sender, bounced_msg, dst_section_pk)
-                    .await?,
-            );
-        }
-
-        Ok(result)
-    }
-
-    pub(crate) async fn handle_anti_entropy_redirect_msg(
-        &self,
-        section_auth: SectionAuthorityProvider,
-        section_signed: KeyedSig,
-        bounced_msg: Bytes,
-        sender: Peer,
-    ) -> Result<Vec<Command>> {
-        debug!(
-            "Anti-Entropy: redirect message received from peer: {}",
-            sender
-        );
-
-        let (bounced_msg, msg_id) = match WireMsg::deserialize(bounced_msg)? {
-            MessageType::System { msg_id, msg, .. } => (msg, msg_id),
-            _ => {
-                warn!("Non System MessageType received at Node in AE response. We do not handle any other type yet");
-                return Ok(vec![]);
-            }
-        };
-
-        // Check if SAP signature is valid
-        let section_signed = SectionAuth {
-            value: section_auth.clone(),
-            sig: section_signed,
-        };
-        if !section_signed.self_verify() {
-            warn!(
-                "Anti-Entropy: failed to verify signature of SAP received in a redirect msg, bounced msg ({:?}) dropped: {:?}",
-                msg_id,bounced_msg
-            );
-            return Ok(vec![]);
-        }
-
-        // Try to find Elders we have in our local records matching the prefix of the
-        // provided SAP, or just use the Elders contained in the provided SAP.
-        // The chosen dst_section_pk will either be the latest we are aware of
-        // if we find a matching prefix in our records, or the genesis_key otherwise.
-        let (dst_section_pk, dst_elders) = match self
-            .network_knowledge
-            .section_by_prefix(&section_auth.prefix())
+            .await?
         {
-            Ok(trusted_sap) => (trusted_sap.section_key(), trusted_sap.peers()),
-            Err(_) => {
-                // In case we don't have the knowledge of that neighbour locally,
-                // let's take the Elders from the provided SAP and genesis key.
-                (
-                    *self.network_knowledge.genesis_key(),
-                    section_signed.peers(),
-                )
-            }
-        };
-
-        // We choose the Elder closest to the dest section key,
-        // just to pick one of them in a random but deterministic fashion.
-        let target_name = XorName::from(PublicKey::Bls(dst_section_pk));
-        let chosen_dst_elder = dst_elders
-            .into_iter()
-            .filter(|elder| section_auth.contains_elder(&elder.name()))
-            .sorted_by(|lhs, rhs| target_name.cmp_distance(&lhs.name(), &rhs.name()))
-            .next();
-
-        if let Some(elder) = chosen_dst_elder {
-            if elder.addr() == sender.addr() {
-                error!(
-                    "Failed to find an alternative Elder to resend msg ({:?}) upon AE-Redirect response.",msg_id
-                );
-                Ok(vec![])
-            } else {
-                trace!("{}", LogMarker::AeResendAfterAeRedirect);
-
-                self.create_or_wait_for_backoff(&elder).await;
-
-                let cmd = self
-                    .send_direct_message(elder, bounced_msg, dst_section_pk)
-                    .await?;
-                Ok(vec![cmd])
-            }
-        } else {
-            warn!(
-                "Anti-Entropy: no trust-worthy elder among incoming SAP {:?} and locally known elders.",
-                section_auth
+            self.write_prefix_map().await;
+            info!(
+                "PrefixMap written to disk with update for prefix {:?}",
+                prefix
             );
+        }
 
-            // For the situation non-elder exists in both incoming and local SAP, send to one of
-            // the incoming elder with the geneis key to trigger AE.
-            if let Some(elder) = section_auth.peers().into_iter().next() {
-                trace!("{}", LogMarker::BounceAfterNewElderNotKnownLocally);
-
-                let cmd = self
-                    .send_direct_message(elder, bounced_msg, *self.network_knowledge.genesis_key())
-                    .await?;
-                Ok(vec![cmd])
-            } else {
-                error!(
-                    "Anti-Entropy: incoming SAP in {:?} doesn't contain any elder! {:?}",
-                    msg_id, section_auth
-                );
-                Ok(vec![])
-            }
+        // If the new SAP's section key is the same as the section key set when the
+        // bounced message was originally sent, we just drop it.
+        if dst_location.section_pk() == Some(dst_section_key) {
+            error!("Dropping bounced msg ({:?}) received in AE-Retry from {} as suggested new dst section key is the same as previously sent: {:?}", msg_id, sender,dst_section_key);
+            Ok(None)
+        } else {
+            Ok(Some((bounced_msg, msg_id)))
         }
     }
 
-    /// Checks AeBackoffCache for backoff, or creates a new instance
+    /// Checks AE-BackoffCache for backoff, or creates a new instance
     /// waits for any required backoff duration
     async fn create_or_wait_for_backoff(&self, peer: &Peer) {
         let mut ae_backoff_guard = self.ae_backoff_cache.write().await;
@@ -281,13 +287,13 @@ impl Core {
         &self,
         original_bytes: Bytes,
         src_location: &SrcLocation,
-        dst_section_pk: &BlsPublicKey,
+        dst_section_key: &BlsPublicKey,
         dst_name: XorName,
         sender: &Peer,
     ) -> Result<Option<Command>> {
-        trace!("Checking for entropy");
         // Check if the message has reached the correct section,
         // if not, we'll need to respond with AE
+        trace!("Checking for entropy");
 
         let our_prefix = self.network_knowledge.prefix().await;
 
@@ -302,13 +308,14 @@ impl Core {
                 .get_closest_or_opposite_signed_sap(&dst_name)
                 .await
             {
-                Some(section_auth) => {
-                    info!("Found a better matching prefix {:?}", section_auth.prefix());
+                Some((signed_sap, section_chain)) => {
+                    info!("Found a better matching prefix {:?}", signed_sap.prefix());
                     let bounced_msg = original_bytes;
                     // Redirect to the closest section
                     let ae_msg = SystemMsg::AntiEntropyRedirect {
-                        section_auth: section_auth.value.into_msg(),
-                        section_signed: section_auth.sig,
+                        section_auth: signed_sap.value.to_msg(),
+                        section_signed: signed_sap.sig,
+                        section_chain,
                         bounced_msg,
                     };
                     let wire_msg = WireMsg::single_src(
@@ -326,17 +333,14 @@ impl Core {
                 }
                 None => {
                     warn!("Our PrefixMap is empty");
-                    // TODO: do we want to reroute some data messages to another seciton here using check_for_better_section_sap_for_data ?
-                    // if not we can remove that function.
-
                     // TODO: instead of just dropping the message, don't we actually need
                     // to get up to date info from other Elders in our section as it may be
                     // a section key we are not aware of yet?
                     // ...and once we acquired new key/s we attempt AE check again?
                     warn!(
-                            "Anti-Entropy: cannot reply with redirect msg for dst_name {:?} and key {:?} to a closest section.",
-                            dst_name, dst_section_pk
-                        );
+                        "Anti-Entropy: cannot reply with redirect msg for dst_name {:?} and key {:?} to a closest section.",
+                        dst_name, dst_section_key
+                    );
 
                     return Err(Error::NoMatchingSection);
                 }
@@ -345,80 +349,67 @@ impl Core {
 
         trace!(
             "Performing AE checks, provided pk was: {:?} ours is: {:?}",
-            dst_section_pk,
+            dst_section_key,
             self.network_knowledge.section_key().await
         );
 
-        if dst_section_pk == &self.network_knowledge.section_key().await {
+        if dst_section_key == &self.network_knowledge.section_key().await {
             trace!("Provided Section PK matching our latest. All AE checks passed!");
             // Destination section key matches our current section key
             return Ok(None);
         }
 
-        /* TODO: get a proof chain rather than whole chain once we have a DAG for our chain.
         let ae_msg = match self
             .network_knowledge
-            .chain()
+            .get_proof_chain_to_current(dst_section_key)
             .await
-            .get_proof_chain_to_current(dst_section_pk)
         {
             Ok(proof_chain) => {
-        */
-        let proof_chain = self.network_knowledge.chain().await;
+                info!("Anti-Entropy: sender's ({}) knowledge of our SAP is outdated, bounce msg for AE-Retry with up to date SAP info.", sender);
 
-        info!("Anti-Entropy: sender's ({}) knowledge of our SAP is outdated, bounce msg for AE-Retry with up to date SAP info.", sender);
+                let signed_sap = self
+                    .network_knowledge
+                    .section_signed_authority_provider()
+                    .await;
 
-        let section_signed_auth = self
-            .network_knowledge
-            .section_signed_authority_provider()
-            .await;
-        let section_auth = section_signed_auth.value;
-        let section_signed = section_signed_auth.sig;
-        let bounced_msg = original_bytes;
-
-        trace!(
-            "Sending AE-Retry with: proofchain last key: {:?} and  section key: {:?}",
-            proof_chain.last_key(),
-            &section_auth.section_key()
-        );
-        trace!("{}", LogMarker::AeSendRetryAsOutdated);
+                trace!(
+                    "Sending AE-Retry with: proofchain last key: {:?} and  section key: {:?}",
+                    proof_chain.last_key(),
+                    &signed_sap.value.section_key()
+                );
+                trace!("{}", LogMarker::AeSendRetryAsOutdated);
 
-        let ae_msg = SystemMsg::AntiEntropyRetry {
-            section_auth: section_auth.into_msg(),
-            section_signed,
-            proof_chain,
-            bounced_msg,
-        };
-        /*
-        }
+                SystemMsg::AntiEntropyRetry {
+                    section_auth: signed_sap.value.to_msg(),
+                    section_signed: signed_sap.sig,
+                    proof_chain,
+                    bounced_msg: original_bytes,
+                }
+            }
             Err(_) => {
                 trace!(
-                    "Anti-Entropy: cannot find dst_section_pk {:?} sent by {} in our chain",
-                    dst_section_pk,
+                    "Anti-Entropy: cannot find dst_section_key {:?} sent by {} in our chain",
+                    dst_section_key,
                     sender
                 );
 
-                let proof_chain = self.network_knowledge.chain().await;
+                let proof_chain = self.network_knowledge.section_chain().await;
 
-                let section_signed_auth = self
+                let signed_sap = self
                     .network_knowledge
                     .section_signed_authority_provider()
                     .await;
-                let section_auth = section_signed_auth.value;
-                let section_signed = section_signed_auth.sig;
-                let bounced_msg = original_bytes;
 
                 trace!("{}", LogMarker::AeSendRetryDstPkFail);
 
                 SystemMsg::AntiEntropyRetry {
-                    section_auth,
-                    section_signed,
+                    section_auth: signed_sap.value.to_msg(),
+                    section_signed: signed_sap.sig,
                     proof_chain,
-                    bounced_msg,
+                    bounced_msg: original_bytes,
                 }
             }
         };
-        */
 
         let wire_msg = WireMsg::single_src(
             &self.node.read().await.clone(),
@@ -433,24 +424,22 @@ impl Core {
         }))
     }
 
-    // generate an AE redirect command for the given message
-    pub(crate) async fn ae_redirect(
+    // Generate an AE redirect command for the given message
+    pub(crate) async fn ae_redirect_to_our_elders(
         &self,
         sender: Peer,
         src_location: &SrcLocation,
         original_wire_msg: &WireMsg,
     ) -> Result<Command> {
-        let section_signed_auth = self
+        let signed_sap = self
             .network_knowledge
             .section_signed_authority_provider()
-            .await
-            .clone();
-        let section_auth = section_signed_auth.value;
-        let section_signed = section_signed_auth.sig;
+            .await;
 
         let ae_msg = SystemMsg::AntiEntropyRedirect {
-            section_auth: section_auth.into_msg(),
-            section_signed,
+            section_auth: signed_sap.value.to_msg(),
+            section_signed: signed_sap.sig,
+            section_chain: self.network_knowledge.section_chain().await,
             bounced_msg: original_wire_msg.serialize()?,
         };
 
@@ -468,51 +457,12 @@ impl Core {
             wire_msg,
         })
     }
-
-    // checks to see if we're actually in the ideal section for this data
-    #[allow(dead_code)]
-    pub(crate) async fn check_for_better_section_sap_for_data(
-        &self,
-        data_name: Option<XorName>,
-    ) -> Option<SectionAuth<SectionAuthorityProvider>> {
-        if let Some(data_name) = data_name {
-            let our_sap = self
-                .network_knowledge
-                .section_signed_authority_provider()
-                .await;
-            trace!("Our SAP: {:?}", our_sap);
-
-            match self
-                .network_knowledge
-                .get_closest_or_opposite_signed_sap(&data_name)
-                .await
-            {
-                Some(better_sap) => {
-                    // Update the client of the actual destination section
-                    trace!(
-                        "We have a better matched section for the data name {:?}",
-                        data_name
-                    );
-                    Some(better_sap)
-                }
-                None => {
-                    trace!(
-                        "We don't have a better matching section for data name {:?}m our SAP: {:?}",
-                        data_name,
-                        our_sap
-                    );
-                    None
-                }
-            }
-        } else {
-            None
-        }
-    }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::elder_count;
     use crate::messaging::{DstLocation, MessageId, MessageType, MsgKind, NodeAuth};
     use crate::routing::{
         create_test_used_space_and_root_storage,
@@ -521,8 +471,9 @@ mod tests {
         network_knowledge::test_utils::{gen_addr, gen_section_authority_provider},
         node::Node,
         routing_api::tests::create_comm,
-        XorName, ELDER_SIZE, MIN_ADULT_AGE,
+        XorName, MIN_ADULT_AGE,
     };
+
     use assert_matches::assert_matches;
     use bls::SecretKey;
     use eyre::{Context, Result};
@@ -542,14 +493,14 @@ mod tests {
         )?;
         let sender = env.core.node.read().await.peer();
         let dst_name = our_prefix.substituted_in(rng.gen());
-        let dst_section_pk = env.core.network_knowledge().section_key().await;
+        let dst_section_key = env.core.network_knowledge().section_key().await;
 
         let command = env
             .core
             .check_for_entropy(
                 msg.serialize()?,
                 &src_location,
-                &dst_section_pk,
+                &dst_section_key,
                 dst_name,
                 &sender,
             )
@@ -571,20 +522,20 @@ mod tests {
         let sender = env.core.node.read().await.peer();
 
         // since it's not aware of the other prefix, it shall redirect us to genesis section/SAP
-        let dst_section_pk = other_pk;
+        let dst_section_key = other_pk;
         let dst_name = env.other_sap.prefix().name();
         let command = env
             .core
             .check_for_entropy(
                 msg.serialize()?,
                 &src_location,
-                &dst_section_pk,
+                &dst_section_key,
                 dst_name,
                 &sender,
             )
-            .await?;
+            .await;
 
-        let msg_type = assert_matches!(command, Some(Command::SendMessage { wire_msg, .. }) => {
+        let msg_type = assert_matches!(command, Ok(Some(Command::SendMessage { wire_msg, .. })) => {
             wire_msg
                 .into_message()
                 .context("failed to deserialised anti-entropy message")?
@@ -592,16 +543,23 @@ mod tests {
 
         assert_matches!(msg_type, MessageType::System{ msg, .. } => {
             assert_matches!(msg, SystemMsg::AntiEntropyRedirect { section_auth, .. } => {
-                assert_eq!(section_auth, env.genesis_sap.into_msg());
+                assert_eq!(section_auth, env.genesis_sap.to_msg());
             });
         });
 
         // now let's insert the other SAP to make it aware of the other prefix
-        assert!(env
-            .core
-            .network_knowledge
-            .prefix_map()
-            .update(env.other_sap.clone(), &env.proof_chain)?);
+        assert!(
+            env.core
+                .network_knowledge()
+                .update_knowledge_if_valid(
+                    env.other_sap.clone(),
+                    &env.proof_chain,
+                    None,
+                    &env.core.node.read().await.name(),
+                    false,
+                )
+                .await?
+        );
 
         // and it now shall give us an AE redirect msg
         // with the SAP we inserted for other prefix
@@ -610,13 +568,13 @@ mod tests {
             .check_for_entropy(
                 msg.serialize()?,
                 &src_location,
-                &dst_section_pk,
+                &dst_section_key,
                 dst_name,
                 &sender,
             )
-            .await?;
+            .await;
 
-        let msg_type = assert_matches!(command, Some(Command::SendMessage { wire_msg, .. }) => {
+        let msg_type = assert_matches!(command, Ok(Some(Command::SendMessage { wire_msg, .. })) => {
             wire_msg
                 .into_message()
                 .context("failed to deserialised anti-entropy message")?
@@ -624,7 +582,7 @@ mod tests {
 
         assert_matches!(msg_type, MessageType::System{ msg, .. } => {
             assert_matches!(msg, SystemMsg::AntiEntropyRedirect { section_auth, .. } => {
-                assert_eq!(section_auth, env.other_sap.value.into_msg());
+                assert_eq!(section_auth, env.other_sap.value.to_msg());
             });
         });
 
@@ -643,14 +601,14 @@ mod tests {
         )?;
         let sender = env.core.node.read().await.peer();
         let dst_name = our_prefix.substituted_in(rng.gen());
-        let dst_section_pk = env.core.network_knowledge().genesis_key();
+        let dst_section_key = env.core.network_knowledge().genesis_key();
 
         let command = env
             .core
             .check_for_entropy(
                 msg.serialize()?,
                 &src_location,
-                dst_section_pk,
+                dst_section_key,
                 dst_name,
                 &sender,
             )
@@ -664,7 +622,7 @@ mod tests {
 
         assert_matches!(msg_type, MessageType::System{ msg, .. } => {
             assert_matches!(msg, SystemMsg::AntiEntropyRetry { ref section_auth, ref proof_chain, .. } => {
-                assert_eq!(section_auth, &env.core.network_knowledge().authority_provider().await.into_msg());
+                assert_eq!(section_auth, &env.core.network_knowledge().authority_provider().await.to_msg());
                 assert_eq!(proof_chain, &env.core.section_chain().await);
             });
         });
@@ -686,14 +644,14 @@ mod tests {
         let dst_name = our_prefix.substituted_in(rng.gen());
 
         let bogus_env = Env::new().await?;
-        let dst_section_pk = bogus_env.core.network_knowledge().genesis_key();
+        let dst_section_key = bogus_env.core.network_knowledge().genesis_key();
 
         let command = env
             .core
             .check_for_entropy(
                 msg.serialize()?,
                 &src_location,
-                dst_section_pk,
+                dst_section_key,
                 dst_name,
                 &sender,
             )
@@ -707,7 +665,7 @@ mod tests {
 
         assert_matches!(msg_type, MessageType::System{ msg, .. } => {
             assert_matches!(msg, SystemMsg::AntiEntropyRetry { ref section_auth, ref proof_chain, .. } => {
-                assert_eq!(*section_auth, env.core.network_knowledge().authority_provider().await.into_msg());
+                assert_eq!(*section_auth, env.core.network_knowledge().authority_provider().await.to_msg());
                 assert_eq!(*proof_chain, env.core.section_chain().await);
             });
         });
@@ -729,7 +687,7 @@ mod tests {
 
             // generate a SAP for prefix0
             let (section_auth, mut nodes, secret_key_set) =
-                gen_section_authority_provider(prefix0, ELDER_SIZE);
+                gen_section_authority_provider(prefix0, elder_count());
             let node = nodes.remove(0);
             let sap_sk = secret_key_set.secret_key();
             let signed_sap = section_signed(sap_sk, section_auth)?;
@@ -760,7 +718,7 @@ mod tests {
 
             // generate other SAP for prefix1
             let (other_sap, _, secret_key_set) =
-                gen_section_authority_provider(prefix1, ELDER_SIZE);
+                gen_section_authority_provider(prefix1, elder_count());
             let other_sap_sk = secret_key_set.secret_key();
             let other_sap = section_signed(other_sap_sk, other_sap)?;
             // generate a proof chain for this other SAP
@@ -794,10 +752,10 @@ mod tests {
             let payload = WireMsg::serialize_msg_payload(&payload_msg)?;
 
             let dst_name = XorName::random();
-            let dst_section_pk = SecretKey::random().public_key();
+            let dst_section_key = SecretKey::random().public_key();
             let dst_location = DstLocation::Node {
                 name: dst_name,
-                section_pk: dst_section_pk,
+                section_pk: dst_section_key,
             };
 
             let msg_id = MessageId::new();
