@@ -33,27 +33,40 @@ impl Core {
         prefix: Prefix,
         elders: BTreeMap<XorName, SocketAddr>,
     ) -> Result<Vec<Command>> {
+        let section_auth = self.network_knowledge().authority_provider().await;
         let section_peers = self.network_knowledge().members();
+
         let elder_candidates = ElderCandidates::new(
             prefix,
             elders.into_iter().map(|(name, addr)| {
                 // Reuse known peers from network_knowledge, in order to preserve connections
-                if let Some(node) = section_peers.get(&name).filter(|node| node.addr() == addr) {
+                if let Some(elder) = section_auth
+                    .get_elder(&name)
+                    .filter(|elder| elder.addr() == addr)
+                {
+                    elder.clone()
+                } else if let Some(node) =
+                    section_peers.get(&name).filter(|node| node.addr() == addr)
+                {
                     node.peer().clone()
                 } else {
                     Peer::new(name, addr)
                 }
             }),
         );
+
         trace!("Received DkgStart for {:?}", elder_candidates);
-        self.dkg_voter
+        let commands = self
+            .dkg_voter
             .start(
                 &self.node.read().await.clone(),
                 session_id,
                 elder_candidates,
                 self.network_knowledge().section_key().await,
             )
-            .await
+            .await?;
+        *self.is_dkg_underway.write().await = true;
+        Ok(commands)
     }
 
     pub(crate) async fn handle_dkg_message(
@@ -108,21 +121,18 @@ impl Core {
         message: DkgMessage,
         sender: XorName,
     ) -> Result<Vec<Command>> {
-        let sender_pk = self
-            .network_knowledge()
-            .section_by_name(&sender)?
-            .section_key();
-        let mut commands = vec![];
-        commands.extend(
-            self.dkg_voter
-                .handle_dkg_history(
-                    &self.node.read().await.clone(),
-                    session_id,
-                    message_history,
-                    sender_pk,
-                )
-                .await?,
-        );
+        let section_key = self.network_knowledge().section_key().await;
+        let mut commands = self
+            .dkg_voter
+            .handle_dkg_history(
+                &self.node.read().await.clone(),
+                session_id,
+                message_history,
+                sender,
+                section_key,
+            )
+            .await?;
+
         commands.extend(
             self.dkg_voter
                 .process_message(
@@ -130,7 +140,7 @@ impl Core {
                     &self.node.read().await.clone(),
                     &session_id,
                     message,
-                    sender_pk,
+                    section_key,
                 )
                 .await?,
         );
@@ -196,6 +206,8 @@ impl Core {
             elder_candidates, failure_set.failed_participants
         );
 
+        *self.is_dkg_underway.write().await = false;
+
         commands.extend(
             self.promote_and_demote_elders_except(&failure_set.failed_participants)
                 .await?,
