@@ -7,6 +7,7 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::super::Core;
+use crate::elder_count;
 use crate::messaging::{
     system::{
         JoinAsRelocatedRequest, JoinAsRelocatedResponse, JoinRejectionReason, JoinRequest,
@@ -19,9 +20,12 @@ use crate::routing::{
     log_markers::LogMarker,
     relocation::RelocatePayloadUtils,
     routing_api::command::Command,
-    Peer, FIRST_SECTION_MAX_AGE, MIN_ADULT_AGE,
+    Peer, SectionAuthUtils, FIRST_SECTION_MAX_AGE, MIN_ADULT_AGE,
 };
 use bls::PublicKey as BlsPublicKey;
+use std::vec;
+
+const FIRST_SECTION_MIN_ELDER_AGE: u8 = 90;
 
 // Message handling
 impl Core {
@@ -90,6 +94,25 @@ impl Core {
             ]);
         }
 
+        if *self.is_dkg_underway.read().await {
+            if self.dkg_voter.has_dkg_underway() {
+                let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Rejected(
+                    JoinRejectionReason::DKGUnderway,
+                )));
+
+                trace!("{}", LogMarker::SendDKGUnderway);
+
+                trace!("Sending {:?} to {}", node_msg, peer);
+                return Ok(vec![
+                    self.send_direct_message(peer, node_msg, our_section_key)
+                        .await?,
+                ]);
+            } else {
+                trace!("is_dkg_underway flag is true, however dkg_voter doesn't have ongoing dkg");
+                *self.is_dkg_underway.write().await = false;
+            }
+        }
+
         if !*self.joins_allowed.read().await {
             debug!(
                 "Rejecting JoinRequest from {} - joins currently not allowed.",
@@ -108,20 +131,47 @@ impl Core {
             ]);
         }
 
-        // During the first section, node shall use ranged age to avoid too many nodes got
-        // relocated at the same time. After the first section got split, nodes shall only
-        // start with age of MIN_ADULT_AGE
+        // During the first section, nodes shall use ranged age to avoid too many nodes getting
+        // relocated at the same time. After the first section splits, nodes shall only
+        // start with an age of MIN_ADULT_AGE
+        let mut section_members = 0;
+
+        // Prefix will be empty for first section
         let (is_age_invalid, expected_age): (bool, u8) = if our_prefix.is_empty() {
-            let section_members = self.network_knowledge.active_members().await.len();
-            // Calculate a value within the range [FIRST_SECTION_MIN_AGE, FIRST_SECTION_MAX_AGE].
-            let expected_age = FIRST_SECTION_MAX_AGE - section_members as u8 * 2;
-            let is_age_invalid = peer.age() == MIN_ADULT_AGE || peer.age() > expected_age;
-            (is_age_invalid, expected_age)
+            let elders = self.network_knowledge.elders().await;
+            section_members = self.network_knowledge.active_members().await.len();
+            // Forces the joining node to be younger than the youngest elder in genesis section
+            // avoiding unnecessary churn.
+
+            // Check if `elder_count()` Elders are already present
+            if elders.len() == elder_count() {
+                // Check if the joining node is younger than the youngest elder and older than
+                // MIN_ADULT_AGE in the first section, to avoid unnecessary churn during genesis.
+                let expected_age = FIRST_SECTION_MIN_ELDER_AGE - section_members as u8 * 2;
+                let is_age_invalid = peer.age() <= MIN_ADULT_AGE || peer.age() > expected_age;
+                (is_age_invalid, expected_age)
+            } else {
+                // Since enough elders haven't joined the first section calculate a value
+                // within the range [FIRST_SECTION_MIN_ELDER_AGE, FIRST_SECTION_MAX_AGE].
+                let expected_age = FIRST_SECTION_MAX_AGE - section_members as u8 * 2;
+                // TODO: avoid looping by ensure can only update to lower non-FIRST_SECTION_MIN_ELDER_AGE age
+                let is_age_invalid = peer.age() != expected_age;
+                (is_age_invalid, expected_age)
+            }
         } else {
+            // Age should be MIN_ADULT_AGE for joining nodes after genesis section.
             let is_age_invalid = peer.age() != MIN_ADULT_AGE;
             (is_age_invalid, MIN_ADULT_AGE)
         };
 
+        trace!(
+            "our_prefix {:?} section_members {:?} expected_age {:?} is_age_invalid {:?}",
+            our_prefix,
+            section_members,
+            expected_age,
+            is_age_invalid
+        );
+
         if !section_key_matches || is_age_invalid {
             if !section_key_matches {
                 trace!("{}", LogMarker::SendJoinRetryNotCorrectKey);
@@ -161,6 +211,18 @@ impl Core {
             ]);
         }
 
+        // If the joining node has aggregated shares from enough elders,
+        // verify the produced auth and accept it into the network.
+        if let Some(response) = join_request.aggregated {
+            if response.verify(&self.section_chain().await) {
+                info!("Handling Online agreement of {:?}", peer);
+                return Ok(vec![Command::HandleNewNodeOnline(response)]);
+                // self
+                // .handle_online_agreement(response.value.clone(), response.sig.clone())
+                // .await;
+            }
+        }
+
         // Require resource signed if joining as a new node.
         if let Some(response) = join_request.resource_proof_response {
             if !self
@@ -193,10 +255,9 @@ impl Core {
             return Ok(vec![cmd]);
         }
 
-        Ok(vec![Command::ProposeOnline {
+        Ok(vec![Command::SendAcceptedOnlineShare {
             peer,
             previous_name: None,
-            dst_key: None,
         }])
     }
 
@@ -327,7 +388,6 @@ impl Core {
         }
 
         let previous_name = Some(details.pub_id);
-        let dst_key = Some(details.dst_key);
 
         if self
             .network_knowledge
@@ -341,10 +401,9 @@ impl Core {
             return Ok(vec![]);
         }
 
-        Ok(vec![Command::ProposeOnline {
+        Ok(vec![Command::SendAcceptedOnlineShare {
             peer,
             previous_name,
-            dst_key,
         }])
     }
 }
