@@ -17,13 +17,6 @@ mod service_msgs;
 mod update_section;
 
 use super::Core;
-use crate::messaging::{
-    data::{ServiceMsg, StorageLevel},
-    signature_aggregator::Error as AggregatorError,
-    system::{NodeCmd, NodeQuery, SystemMsg},
-    DstLocation, MessageId, MessageType, MsgKind, NodeMsgAuthority, SectionAuth, ServiceAuth,
-    WireMsg,
-};
 use crate::routing::{
     log_markers::LogMarker,
     messages::{NodeMsgAuthorityUtils, WireMsgUtils},
@@ -33,6 +26,16 @@ use crate::routing::{
     Error, Event, MessageReceived, Peer, Result, UnnamedPeer, MIN_LEVEL_WHEN_FULL,
 };
 use crate::types::{Chunk, Keypair, PublicKey};
+use crate::{
+    messaging::{
+        data::{ServiceMsg, StorageLevel},
+        signature_aggregator::Error as AggregatorError,
+        system::{NodeCmd, NodeQuery, SystemMsg},
+        AuthorityProof, DstLocation, MessageId, MessageType, MsgKind, NodeMsgAuthority,
+        SectionAuth, ServiceAuth, WireMsg,
+    },
+    routing::core::DkgSessionInfo,
+};
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
 use rand::rngs::OsRng;
@@ -282,6 +285,8 @@ impl Core {
         sender: Peer,
         known_keys: Vec<BlsPublicKey>,
     ) -> Result<Vec<Command>> {
+        self.network_knowledge().merge_connections([&sender]).await;
+
         let src_name = msg_authority.name();
         trace!("Handling non blocking message");
         match node_msg {
@@ -437,8 +442,8 @@ impl Core {
                 sig_share,
             } => {
                 if self.is_not_elder().await {
-                    trace!("Dropping Propose msg from {}: {:?}", sender, msg_id);
-                    return Ok(vec![]);
+                    trace!("Adult handling a Propose msg from {}: {:?}", sender, msg_id);
+                    // return Ok(vec![]);
                 }
 
                 trace!("Handling msg: Propose from {}: {:?}", sender, msg_id);
@@ -454,7 +459,16 @@ impl Core {
                 if !elders.contains_key(&self.node.read().await.name()) {
                     return Ok(vec![]);
                 }
-
+                if let NodeMsgAuthority::Section(authority) = msg_authority {
+                    let _existing = self.dkg_sessions.write().await.insert(
+                        session_id,
+                        DkgSessionInfo {
+                            prefix,
+                            elders: elders.clone(),
+                            authority,
+                        },
+                    );
+                }
                 self.handle_dkg_start(session_id, prefix, elders).await
             }
             SystemMsg::DkgMessage {
@@ -481,15 +495,12 @@ impl Core {
             SystemMsg::DkgNotReady {
                 message,
                 session_id,
-            } => {
-                let src = sender.name();
-                Ok(self.handle_dkg_not_ready(
-                    sender,
-                    message,
-                    session_id,
-                    self.network_knowledge.section_by_name(&src)?.section_key(),
-                ))
-            }
+            } => Ok(self.handle_dkg_not_ready(
+                sender,
+                message,
+                session_id,
+                self.network_knowledge.section_key().await,
+            )),
             SystemMsg::DkgRetry {
                 message_history,
                 message,
@@ -600,6 +611,91 @@ impl Core {
                 )
                 .await
             }
+            SystemMsg::DkgSessionUnknown {
+                session_id,
+                message,
+            } => {
+                if let Some(session_info) = self.dkg_sessions.read().await.get(&session_id).cloned()
+                {
+                    let DkgSessionInfo {
+                        prefix,
+                        elders,
+                        authority: section_auth,
+                    } = session_info;
+                    let message_cache = self.dkg_voter.get_cached_messages(&session_id);
+                    trace!(
+                        "Sending DkgSessionInfo {{ {:?}, ... }} to {}",
+                        &session_id,
+                        &sender
+                    );
+                    Ok(vec![Command::PrepareNodeMsgToSend {
+                        msg: SystemMsg::DkgSessionInfo {
+                            session_id,
+                            elders,
+                            prefix,
+                            section_auth,
+                            message_cache,
+                            message,
+                        },
+                        dst: DstLocation::Node {
+                            name: sender.name(),
+                            section_pk: self.network_knowledge.section_key().await,
+                        },
+                    }])
+                } else {
+                    warn!("Unknown DkgSessionInfo: {:?} requested", &session_id);
+                    Ok(vec![])
+                }
+            }
+            SystemMsg::DkgSessionInfo {
+                session_id,
+                prefix,
+                elders,
+                message_cache,
+                section_auth,
+                message,
+            } => {
+                let mut commands = vec![];
+                // Reconstruct the original DKG start message and verify the section signature
+                let payload = WireMsg::serialize_msg_payload(&SystemMsg::DkgStart {
+                    session_id,
+                    prefix,
+                    elders: elders.clone(),
+                })?;
+                let auth = section_auth.clone().into_inner();
+                if self.network_knowledge.section_key().await == auth.sig.public_key {
+                    if let Err(err) = AuthorityProof::verify(auth, payload) {
+                        error!("Error verifying signature for DkgSessionInfo: {:?}", err);
+                        return Ok(commands);
+                    } else {
+                        trace!("DkgSessionInfo signature verified");
+                    }
+                } else {
+                    warn!(
+                        "Cannot verify DkgSessionInfo: {:?}. Unknown key: {:?}!",
+                        &session_id, auth.sig.public_key
+                    );
+                    warn!(
+                        "Chain: {:?}",
+                        self.network_knowledge().section_chain().await
+                    );
+                    return Ok(commands);
+                }
+                let _existing = self.dkg_sessions.write().await.insert(
+                    session_id,
+                    DkgSessionInfo {
+                        prefix,
+                        elders: elders.clone(),
+                        authority: section_auth,
+                    },
+                );
+                commands.extend(self.handle_dkg_start(session_id, prefix, elders).await?);
+                commands.extend(
+                    self.handle_dkg_retry(session_id, message_cache, message, sender.name())
+                        .await?,
+                );
+                Ok(commands)
+            }
         }
     }
 
@@ -728,10 +824,14 @@ impl Core {
         .await
         {
             Ok(section_auth) => {
+                info!("Successfully aggregated message");
                 *msg_authority = NodeMsgAuthority::Section(section_auth);
                 Ok(false)
             }
-            Err(AggregatorError::NotEnoughShares) => Ok(true),
+            Err(AggregatorError::NotEnoughShares) => {
+                info!("Not enough shares to aggregate received message");
+                Ok(true)
+            }
             Err(err) => {
                 error!("Error accumulating message at dst: {:?}", err);
                 Err(Error::InvalidSignatureShare)
