@@ -35,7 +35,13 @@ pub(crate) use section_authority_provider::SectionAuthorityProvider;
 pub(crate) use section_peers::SectionPeers;
 use secured_linked_list::SecuredLinkedList;
 use serde::Serialize;
-use std::{collections::BTreeSet, convert::TryInto, iter, net::SocketAddr, sync::Arc};
+use std::{
+    collections::{BTreeMap, BTreeSet},
+    convert::TryInto,
+    iter,
+    net::SocketAddr,
+    sync::Arc,
+};
 use tokio::sync::RwLock;
 use xor_name::{Prefix, XorName};
 
@@ -123,7 +129,7 @@ impl NetworkKnowledge {
         };
 
         // At this point we know the prefix map corresponds to the correct genesis key,
-        // let's make sure the prefix map contains also our own pefix and SAP,
+        // let's make sure the prefix map contains also our own prefix and SAP,
         if let Err(err) = prefix_map.update(signed_sap.clone(), &chain) {
             debug!("Failed to update NetworkPrefixMap with SAP {:?} and chain {:?} upon creating new NetworkKnowledge intance: {:?}", signed_sap, chain, err);
         }
@@ -501,6 +507,11 @@ impl NetworkKnowledge {
         &self.section_peers
     }
 
+    /// Returns the elders of our section
+    pub(super) async fn elders(&self) -> Vec<Peer> {
+        self.authority_provider().await.elders_vec()
+    }
+
     /// Returns members that are either joined or are left but still elders.
     pub(super) async fn active_members(&self) -> Vec<Peer> {
         let mut active_members = vec![];
@@ -547,6 +558,28 @@ impl NetworkKnowledge {
             .map(|info| info.peer().clone())
     }
 
+    /// Merge the connections from some source peers into our state.
+    ///
+    /// Peers are held in `signed_sap` and `section_peers`, and we match relevant peers with both.
+    pub(super) async fn merge_connections(&self, sources: impl IntoIterator<Item = &Peer>) {
+        let sources: BTreeMap<_, _> = sources
+            .into_iter()
+            .map(|peer| (peer.addr(), peer))
+            .collect();
+
+        for elder in self.signed_sap.read().await.elders() {
+            if let Some(source) = sources.get(&elder.addr()) {
+                elder.merge_connection(source).await;
+            }
+        }
+
+        for node in self.section_peers.iter() {
+            if let Some(source) = sources.get(&node.addr()) {
+                node.peer().merge_connection(source).await;
+            }
+        }
+    }
+
     // Tries to split our section.
     // If we have enough mature nodes for both subsections, returns the SectionAuthorityProviders
     // of the two subsections. Otherwise returns `None`.
